/*
 * World.hpp
 *
 *  Created on: Feb 3, 2010
 *      Author: crueger
 */

#ifndef WORLD_HPP_
#define WORLD_HPP_

#include <boost/thread.hpp>
#include <map>
#include <vector>

#include "Patterns/Observer.hpp"
#include "Patterns/Cacheable.hpp"

// forward declarations
class periodentafel;
class MoleculeListClass;
class atom;
class AtomDescriptor;
class AtomDescriptor_impl;

class World : public Observable
{
friend class AtomDescriptor_impl;
public:

  /***** getter and setter *****/
  periodentafel* getPeriode();
  atom* getAtom(AtomDescriptor descriptor);
  std::vector<atom*> getAllAtoms(AtomDescriptor descriptor);
private:
  periodentafel *periode;
  std::map<int,atom*> atoms;


  /***** singleton Stuff *****/
public:
  static World* get();
  static void destroy();
  static World* reset();

private:
  World();
  virtual ~World();

  static World *theWorld;
  // this mutex only saves the singleton pattern...
  // use other mutexes to protect internal data as well
  // this mutex handles access to the pointer, not to the object!!!
  static boost::mutex worldLock;

  /*****
   * some legacy stuff that is include for now but will be removed later
   *****/
public:
  MoleculeListClass *getMolecules();

  // functions used for the WorldContent template mechanism
  void registerAtom(atom *theAtom);
  void unregisterAtom(atom *theAtom);
private:
  MoleculeListClass *molecules;

  // this is needed to assign unique IDs to atoms... so far
  // IDs are not assigned upon Atom creation, so we cannot query the ID
  // during construction. By using the dummy ID we can make sure all atoms
  // are actually stored in the map and don't overwrite each other.
  int dummyId;
};

#endif /* WORLD_HPP_ */
